@@ -1,10 +1,11 @@
 from pyterrier.java import mavenresolver
 from pyterrier.java._utils import JavaInitializer, init, started, parallel_init, parallel_init_args, required, required_raise, before_init, autoclass, cast, legacy_init, register_config, Configuration, JavaClasses
-from pyterrier.java._core import CoreJavaInit, configure, redirect_stdouterr, bytebuffer_to_array, J, add_jar, add_package, set_memory_limit, set_redirect_io, add_option, set_log_level, set_java_home
+from pyterrier.java._core import CoreJavaInit, ColabJavaInit, configure, redirect_stdouterr, bytebuffer_to_array, J, add_jar, add_package, set_memory_limit, set_redirect_io, add_option, set_log_level, set_java_home
 
 __all__ = [
     'add_jar', 'add_option', 'add_package', 'autoclass', 'before_init', 'bytebuffer_to_array', 'cast', 'config',
     'configure', 'init', 'J', 'JavaClasses', 'mavenresolver', 'parallel_init', 'parallel_init_args',
     'redirect_stdouterr', 'required', 'required_raise', 'set_log_level', 'set_memory_limit', 'set_redirect_io',
-    'started', 'legacy_init', 'JavaInitializer', 'CoreJavaInit', 'set_java_home', 'register_config', 'Configuration',
+    'started', 'legacy_init', 'JavaInitializer', 'CoreJavaInit', 'ColabJavaInit', 'set_java_home', 'register_config', 
+    'Configuration',
 ]
